/**
 * Copyright (c) 2014-2019 Zuoyebang, All rights reserved.
 * @fileoverview bridge
 * @author carl | carl@zuoyebang.com
 * @version 1.0  | 2019-03-27 | carl    // 初始版本。
 *
 * @method invokeMethod(name, args, cb)    // fe 调用 win 移动端 
 *   @param name          // 参数
 *   @return no         // 返回：无。
 *   @description  在win端使用cb时，必须时args有参数并且时object类型时才可以，
 *                 是跟win端约定而成的，会在你args对象中args.length时插入一个'callback'参数用于win端同学来识别
 *                 当有cb时，会在执行完invokeMethod后主动调用一次addMethod来执行你的cb回调
 *
 * @method addMethod(name, method, once)     //  win 调用 fe, 移动端不需要这个方法直接调用
 *   @param name {String}    // 。
 *   @param once 为默认false, 为true时只能被端调用一次    // 。
 *   @return No                   // 返回：无。
 *
 * @method removeMethod(name, method)  //  fe 调用 win 用来取消监听方法
 *   @param String    //
 *   @return no                    //
 *
 * @description    // 附加说明。
 *   1) 为了方便前端对win 和 移动端 调用方式统一并且要基于win端的风格，所以做了简单包装。
 *      参数 name 对应 移动端的 action
 *      参数 args 对应 移动端的 args
 *      参数 cb 对应 移动端的 callback
 *
 * @example    // 典型的调用示例。
 *    
 *    require('@/common/bridge')
 *    // 移动端使用
 *    Bridge.invokeMethod('actionName', {
 *      arg1: '',
 *      arg2: ''
 *    }, function(){
 *      // app 回调。
 *    };
 *
 *    // is_login: 判断用户是否登录。
 *    Bridge.invokeMethod('is_login', function(res){});    // res: {isLogin: boolean}
 *    
 *    // login: 跳转登录页面。
 *    Bridge.invokeMethod('login');
 *
 *    // openWindow: 打开一个指定的web页面。
 *    Bridge.invokeMethod('openWindow', {
 *      url: string    // 指定页面的完整url，示例：http://www.zybang.com/zuoyebang/fudao/index.html 
 *    });
 *
 *    // win端使用方式
 *   Bridge.invokeMethod(name,jsonObj);
 *   Bridge.invokeMethod(name,jsonObj, function(){});
 *   Bridge.addMethod(name,function(res){});
 *   Bridge.addMethod(name,callWindows);
 *   function callWindows(){}
 *   Bridge.removeMethod("jsOpenLiveroomError", this.enterClassErrorHandler);
 * 
 *       winBridge.addMethod("onOpen", function(){
            console.log('onOpen成功了')
        }, true)
 * 
 *
 */

// 没有提供umd的方式，认为没有必要
var qCefClientObj = window.QCefClient || null //主讲端注入的全局对象
var isLocal = /(localhost|127.0.0.1)/.test(location.origin), __cb__ = null;
var hybridFn = function(){}

//添加一层监听
// var _callbacks = {};
// var _watchMethod = null;
// var filterMethod = function (event) {
//   console.info("--------监听win端调用--------: ",event);
//   _callbacks[event.name] && _callbacks[event.name].forEach(e=>e&&e(event));
//   _watchMethod && _watchMethod(event);
// };

class Bridge {
    constructor() {
        if (this.isBrid()) {
            hybridFn = this._hybrid()
        }
        this.onceMethod = []
    }

   

    isBrid (){
        if (window.navigator.userAgent.match(/airclass|homework|flipped/g)) {
            return true
        }else {
            return false 
        }
    }

    invokeMethod(name, args, cb){
        console.log('-1--')
        console.log(name)
        console.log(args)
        console.log(cb)
        console.log('-1--')
        if (this.isBrid()) {
            hybridFn(name, args, cb)
        } else {
            if (!qCefClientObj) {
                console.info(`开发环境跳过触发win端方法=${name},参数=${JSON.stringify(args)}`);
                return false;
            }
            var argString;
            if (name && typeof name == 'string') {
                if (args) {
                    if (typeof args == "object") {

                        if (cb && typeof cb == 'function') {
                            args[`${Object.keys(args).length}`] = 'callback'
                        }

                        argString = JSON.stringify(args);
                        qCefClientObj.invokeMethod(name, argString);

                        if (cb && typeof cb == 'function') {
                            this.addMethod(name, cb)
                        }
                    }
                    else if(typeof args === 'string') {
                        console.log("###invokeMethod-name=" + name);
                        qCefClientObj.invokeMethod(name, args);
                    }
                    else {
                        throw new Error('[bridge Exception]: Invalid arguments!');
                    }
                } else {
                    qCefClientObj.invokeMethod(name);
                }
            } else {
                throw new Error('[bridge Exception]: Invalid arguments!');
            }
        }
        
    }

    addMethod(name, method, once=false) {
        if (this.isBrid()) {
            if (name && typeof name == 'string' && method && typeof method == 'function') {
                window[name] = method;
            }else {
                throw new Error('[bridge Exception]: addEventListener Invalid arguments!');
            }
        }else{
            if (name && typeof name == 'string' && method && typeof method == 'function') {
                if (qCefClientObj) {
                    console.log("###addMethod-name=" + name,new Date());
                    
                    if (once) {
                        if (localStorage.getItem('addmethod')) {
                            console.log('remove  addmethod ')
                            qCefClientObj.removeEventListener(name, method);
                            //  var getTempStr = localStorage.getItem('addmethodobj')
                            // this.onceMethod = JSON.parse(getTempStr)
                            // this.onceMethod.some((val, index) => {
                            //     if (name === val.name && val.count === 0) {
                            //         qCefClientObj.removeEventListener(val.name, val.method);
                            //         val.count = 1
                            //     }
                            // })
                        }else {
                            
                            qCefClientObj.addEventListener(name, method);
                            // 无这种情况
                            // this.onceMethod.some((item, index) => {
                            //     if (item.name != name) {
                            //         this.onceMethod.push({
                            //             key: name,
                            //             fn: method,
                            //             count: 0
                            //         })
                            //         return true
                            //     }
                            // })
                            // var tempObj = this.onceMethod;
                            // localStorage.setItem('addmethodobj', JSON.stringify(tempObj))

                            localStorage.setItem('addmethod', name)
                        }
                    }else{
                        qCefClientObj.addEventListener(name, method);
                    }
                }else{
                    //本地调试 不支持一次调用
                    window.devWin = window.devWin ? window.devWin : {};
                    window.devWin[name] = method;
                }
            } else {
                throw new Error('[bridge Exception]: addEventListener Invalid arguments!');
            }
        }
    }

    removeMethod(name,method){
        if (this.isBrid()) {
            return false
        }
        if (name && typeof name == 'string' && method && typeof method == 'function') {
            if (qCefClientObj) {
                console.log("###removeEventListener-name=" + name);
                qCefClientObj.removeEventListener(name, method);
            }
        } else {
            throw new Error('[bridge Exception]: removeEventListener Invalid arguments!');
        }
    }

    _hybrid() {
        var bridge,
        protocol = !isLocal ? 'iknowhybrid://' : '/hybridaction/',
        callbackCount = 0;
        let self = this;
    
        function createBridge(src) {
            var bridge = document.createElement('iframe');
            bridge.setAttribute('style', 'display:none;');
            bridge.setAttribute('height', '0px');
            bridge.setAttribute('width', '0px');
            bridge.setAttribute('frameborder', '0');
            bridge.setAttribute('src', 'about:blank');
        
            document.documentElement.appendChild(bridge);
            return bridge;
        }
    
        function buildUrl(action, params, callback) {
            var url = protocol + action + '?';
            var id = self._guid()
            url += 'data=' + encodeURIComponent(JSON.stringify(params || {}));
            if (callback) {
                var callbackName = '__cb__' + id + '__' + callbackCount++;
                // isLocal
                isLocal && (__cb__ = callbackName);
                window[callbackName] = function () {
                var args = arguments;
                if (isLocal) {
                    var result = null;
                    try {
                    result = JSON.parse(self.contentDocument.body.children[0].textContent);
                    args = [result] || args;
                    } catch (e) {
                    console.log(e);
                    }
                }
                callback && callback.apply(window, args);
                }
                url += '&__callback__=' + callbackName;
            }
            return url;
        }
        return function (action, params, callback) {
            bridge = createBridge();
            if (params && Object.prototype.toString.call(params) == '[object Function]') {
                callback = params;
                params = {};
            }
            var url = buildUrl(action, params, callback);
        
            isLocal && (bridge.onload = window[__cb__]);
            __cb__ = null;
        
            bridge.setAttribute('src', url);
        }
    };
    

    _guid() {
        return 'xxxxxxxx_xxxx_4xxx_yxxx_xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
        var r = Math.random() * 16 | 0, v = c == 'x' ? r : (r & 0x3 | 0x8);
        return v.toString(16);
        });
    }

}

(function(){
    window.winBridge = new Bridge();
})()