/**
 * Copyright (c) 2014-2016 Zuoyebang, All rights reserved.
 * @fileoverview 与windows端交互api
 * @author chuxueyang | chuxueyang@zuoyebang.com
 * @version 1.0 | 2016-05-26 | chuxueyang    // 初始版本。
 * @version 1.1 | 2018-05-20 | liujianwei    // 添加removeMethod方法，添加日志
 *
 * @method invokeMethod(name, args)    // 方法：调用windows端方法。
 *   @param name{String}               // 参数：对应windows端的方法名。onXXX开头,驼峰写法  比如：onClose
 *   @param args {Object}              // 参数：传给windows端的参数 json结构索引key以0开始 (可选) 例：{0：A,1,B,2:C}。
 *
 * @method addMethod(name, method)    // 方法：向windows端注册自定义方法。
 *   @param name{String}              // 参数：注册的函数名。jsXXX,驼峰写法，比如：jsCloseDiaolog（必选）
 *   @param method {Function}         // 参数：注册的函数（必选）
 *
 * @example    // 典型的调用示例。
 *  var winBridge = require('airclass:widget/winBridge/winBridge.js'),
 *    jsonObj = {0:goCourseId.,1:goLessonId,2:courseName};
 *
 *  winBridge.invokeMethod(name,jsonObj);
 *
 *  winBridge.addMethod(name,callWindows);
 *
 *  function callWindows(){
 *
 *  }
 *
 */

var qCefClientObj = null; //主讲端注入的全局对象

if (typeof (QCefClient) == 'undefined') {
    console.log('未找到主讲端注入对象！');
} else {
    qCefClientObj = QCefClient;
}
//添加一层监听
var _callbacks = {};
var _watchMethod = null;
var filterMethod = function (event) {
  console.info("--------监听win端调用--------: ",event);
  _callbacks[event.name] && _callbacks[event.name].forEach(e=>e&&e(event));
  _watchMethod && _watchMethod(event);
};

var winBridge = {
    invokeMethod: function (name, args) {
        if (!qCefClientObj) {
            console.info(`开发环境跳过触发win端方法=${name},参数=${JSON.stringify(args)}`);
            return false;
        }
        var argString;

        if (name && typeof name == 'string') {
            if (args) {
                if (typeof args == "object") {
                    for (var i in args) {
                        args[i] = !!args[i] ? args[i].toString() : "";
                    }
                    console.log("###invokeMethod-name=" + name);
                    argString = JSON.stringify(args);
                    console.log(argString);
                    qCefClientObj.invokeMethod(name, argString);

                }
                else if(typeof args === 'string') {
                  console.log("###invokeMethod-name=" + name);
                  console.log(args);
                  qCefClientObj.invokeMethod(name, args);
                }
                else {
                    throw new Error('[winBridge Exception]: Invalid arguments!');
                }
            } else {
                qCefClientObj.invokeMethod(name);
            }
        } else {
            throw new Error('[winBridge Exception]: Invalid arguments!');
        }
    },
    addMethod: function (name, method) {
        if (name && typeof name == 'string' && method && typeof method == 'function') {
            if (qCefClientObj) {
                console.log("###addMethod-name=" + name,new Date());
              // qCefClientObj.addEventListener(name, method);
              _callbacks[name] = _callbacks[name] ?_callbacks[name] : [];
              _callbacks[name].push(method);
                qCefClientObj.addEventListener(name, filterMethod);
            }else{
              //本地调试
              window.devWin = window.devWin ? window.devWin : {};
              _callbacks[name] = _callbacks[name] ?_callbacks[name] : [];
              _callbacks[name].push(method);
              window.devWin[name] = e=>{
                filterMethod(Object.assign({name:name},e));
              };
            }
        } else {
            throw new Error('[winBridge Exception]: addEventListener Invalid arguments!');
        }
    },
    removeMethod:function(name,method){
        if (name && typeof name == 'string' && method && typeof method == 'function') {
            if (qCefClientObj) {
                console.log("###removeEventListener-name=" + name);
                // qCefClientObj.removeEventListener(name, method);
                _callbacks.hasOwnProperty(name) && delete _callbacks[name];
                qCefClientObj.removeEventListener(name, filterMethod);
            }
        } else {
            throw new Error('[winBridge Exception]: removeEventListener Invalid arguments!');
        }
    },
    addWatcher : function (fn) {
      if(typeof fn !== "function")return;
      _watchMethod = fn;
    }
};

// export default winBridge;
window.winBridge = winBridge;
